#
# level module
# level.py
#

import random
import pygame
from pygame.locals import *
import spaceship
import vector

TILE_DICT = {
# ID: ["filename.png",CanPass,CanShoot,LoadedTile]
0: ["grass.png",True,True,None],
1: ["hfence.png",False,False,None],
2: ["vfence.png",False,False,None],
3: ["cfence.png",False,False,None],
4: ["flowers.png",False,True,None],
}
TILE_SIZE = 32
TYPE_PASS = 1
TYPE_SHOOT = 2

LINE_OF_SIGHT_SPEED = TILE_SIZE / 2
LINE_OF_SIGHT_SIZE = 8

def load_tiles():
	global TILE_DICT
	for i in TILE_DICT:
		TILE_DICT[i][3] = pygame.image.load(TILE_DICT[i][0]).convert_alpha()

class level():
	def __init__(self,world,levelfile):
		global TILE_DICT
		global TILE_SIZE
		self._world = world
		tmp = __import__(levelfile)
		self._data = tmp.data
		self._time_left = self._data["time"]
		self._enemies = []
		for enemy in self._data["enemies"]:
			self._enemies.append({"alive":False,"timeleft":enemy[1]})
		self._canshoot = [[]]
		self._canpass = [[]]
		self._tiles = [[]]
		fo = file(self._data["map"],"rt")
		chr = fo.read(1)
		while chr:
			if chr != "\n":
				info = TILE_DICT[int(chr)]
				self._canpass[-1].append(info[1])
				self._canshoot[-1].append(info[2])
				self._tiles[-1].append(info[3])
			else:
				self._canpass.append([])
				self._canshoot.append([])
				self._tiles.append([])
			chr = fo.read(1)
		# generate single image
		self._img = pygame.Surface((len(self._tiles[0]) * TILE_SIZE,len(self._tiles) * TILE_SIZE),0,32).convert()
		for y in range(0,self._img.get_height(),TILE_SIZE):
			for x in range(0,self._img.get_width(),TILE_SIZE):
				self._img.blit(self._tiles[y / TILE_SIZE][x / TILE_SIZE],(x,y))
		
	def draw(self,pos):
		self._world._screen.blit(self._img,pos.get())
	
	def collide(self,pos,size,type):
		global TILE_SIZE
		size -= 1.5 # avoid constant enemy collision and one tile pass bug
		lx = pos._x - size / 2.0
		ly = pos._y - size / 2.0
		xlist = [lx / TILE_SIZE,(lx + size) / TILE_SIZE]
		ylist = [ly / TILE_SIZE,(ly + size) / TILE_SIZE]
		tocheck = self._canpass
		if type == TYPE_SHOOT:
			tocheck = self._canshoot
		for y in ylist:
			for x in xlist:
				if False == tocheck[int(y)][int(x)]:
					return True
		return False
	
	def process(self,time_passed):
		self._time_left -= time_passed / 1000.0
		if self._time_left <= 0:
			return False
		for i in range(len(self._enemies)):
			if not self._enemies[i]["alive"]:
				self._enemies[i]["timeleft"] -= time_passed / 1000.0
				if self._enemies[i]["timeleft"] <= 0:
					self._enemies[i]["timeleft"] = self._data["enemies"][i][2]
					self._enemies[i]["alive"] = True
					id = self._world.add_obj(spaceship.spaceship(self._world,self._random_pos(),(i,self._data["enemies"][i][0])))
					self._world._objs[id].set_id(id)
		return True
	
	def _random_pos(self):
		tx = random.randint(0,len(self._canpass[0]) - 1)
		ty = random.randint(0,len(self._canpass) - 1)
		while False == self._canpass[ty][tx]:
			tx = random.randint(0,len(self._canpass[0]) - 1)
			ty = random.randint(0,len(self._canpass) - 1)
		return vector.twin((tx + 0.5) * TILE_SIZE,(ty + 0.5) * TILE_SIZE)
	
	def enemy_died(self,eobj):
		eid = eobj._drop_info[0]
		self._world._score += 10
		if self._data["enemies"][eid][0] == 1:
			self._world._score += 40
		self._enemies[eid]["alive"] = False
	
	def has_line_of_sight(self,pos1,pos2,maxdist):
		heading = pos2 - pos1
		pos = vector.twin(pos1.get())
		dist = heading.len()
		if dist > maxdist:
			return False
		heading.normalize()
		speed = LINE_OF_SIGHT_SPEED
		while dist > 0:
			if self.collide(pos,LINE_OF_SIGHT_SIZE,TYPE_SHOOT):
				return False
			move = heading * speed
			pos = pos + move
			dist = dist - move.len()
		return True
