#
# player object and logic
# player.py
#

import pygame
from pygame.locals import *
import gameobj
from level import TYPE_PASS
import vector
import shot

class player(gameobj.gameobj):
	def __init__(self,world):
		gameobj.gameobj.__init__(self,world)
		self._sprite = pygame.image.load("player.png").convert_alpha()
		self._rotated_sprite = self._sprite
		self._life_offset = vector.twin(self._sprite.get_width() / 2.0,self._sprite.get_height() / 2.0)
		self._speed = 70.0
		self._life = 100.0
		self._maxlife = 100.0
		self.size = 32
		self._weapons = {
		shot.SIMPLE:{"charge": 160.0,"rate": 40,"cost": 20, "maxcharge": 160.0 },
		shot.POWER:{"charge": 100.0,"rate": 15,"cost": 50, "maxcharge": 100.0 },
		shot.GRENADE:{"charge": 100.0,"rate": 5,"cost": 100, "maxcharge": 100.0 }
		}
		self._lifebar = pygame.Surface((self.size,3),0,32).convert()
		self._prepare_lifebar()
		# generate rotated sprites
		self._sprite_dict = self._generate_sprites()

	def _generate_sprites(self):
		sprite_dict = {}
		v = vector.twin(1.0,1.0)
		v.normalize()
		diag_value = v._x
		del v
		splist = [
			[0.0,-1.0,0],
			[-diag_value,-diag_value,45],
			[-1.0,0.0,90],
			[-diag_value,diag_value,135],
			[0.0,1.0,180],
			[diag_value,diag_value,225],
			[1.0,0.0,270],
			[diag_value,-diag_value,315]
			]
		for pair in splist:
			if not pair[0] in sprite_dict:
				sprite_dict[pair[0]] = {}
			sprite_dict[pair[0]][pair[1]] = pygame.transform.rotate(self._sprite,pair[2])
		return sprite_dict
		
	def _prepare_lifebar(self):
		self._lifebar.fill((0,0,0))
		pygame.draw.line(self._lifebar,(255,128,128),(1,1),(self._life / self._maxlife * (self.size - 2),1))

	def draw(self,pos):
		if not self._heading == vector.twin(0,0):
			self._rotated_sprite = self._sprite_dict[self._heading._x][self._heading._y]
		draw_offset = vector.twin(self._rotated_sprite.get_width() / 2.0,self._rotated_sprite.get_height() / 2.0)
		self._world._screen.blit(self._rotated_sprite,
			(pos - draw_offset).get())
		self._world._screen.blit(self._lifebar,(pos - self._life_offset - vector.twin(0,self._lifebar.get_height())).get())
	
	def process(self,time_passed):
		gameobj.gameobj.process(self,time_passed)
		# recharge weapons
		for w in self._weapons:
			self._weapons[w]["charge"] += time_passed * self._weapons[w]["rate"] / 1000.0
			if self._weapons[w]["charge"] > self._weapons[w]["maxcharge"]:
				self._weapons[w]["charge"] = self._weapons[w]["maxcharge"]
	
	def collided(self,time_passed):
		self.process(-time_passed)
		# attempt a vertical slide
		old_heading = vector.twin(self._heading.get())
		self._heading._x = 0
		self._heading.normalize()
		self.process(time_passed)
		if self._world._level.collide(self._pos,self.size,TYPE_PASS):
			self.process(-time_passed)
			# attempt a horizontal slide
			self._heading = old_heading
			self._heading._y = 0
			self._heading.normalize()
			self.process(time_passed)
			if self._world._level.collide(self._pos,self.size,TYPE_PASS):
				self.process(-time_passed)
	
	def shoot(self,type,pos):
		if self._weapons[type]["charge"] >= self._weapons[type]["cost"]:
			id = self._world.add_obj(shot.shot(self._world,type,self._pos,pos))
			self._world._objs[id].set_id(id)
			self._world._objs[id].set_owner(self)
			self._weapons[type]["charge"] -= self._weapons[type]["cost"]
	
	def check_hit(self,shotobj):
		if shotobj._owner == self:
			return False
		if (self._pos - shotobj._pos).len() < ((self.size + shotobj.size) / 2.0):
			self._life -= shotobj._dmg
			self._prepare_lifebar()
			return True
		return False
